// Copyright (2013) Jann Horn <jann@thejh.net>

HEADER #include <sys/types.h>
#include <string.h>

static inline char hexchar(unsigned char c) {
  if (c >= 10) return c+'a'-10;
  return c+'0';
}

// If `in_len` is -1, it means that `in` is a null-terminated string.
// `out` must have space for `in_len*2+1` bytes.
PUBLIC_FN void hex_encode(char *in, ssize_t in_len, char *out) {
  if (in_len == -1) in_len = strlen(in);
  assert(in_len >= 0);
  char *in_end = in+in_len;
  while (in < in_end) {
    unsigned char c = *(unsigned char *)(in++);
    *(out++) = hexchar(c>>4);
    *(out++) = hexchar(c&0xf);
  }
  *(out++) = '\0';
}

static inline unsigned char unhexchar(char c) {
  if (c >= '0' && c <= '9') return c-'0';
  if (c >= 'a' && c <= 'f') return c-'a'+10;
  if (c >= 'A' && c <= 'F') return c-'A'+10;
  return 255;
}

// Returns number of decoded bytes or -1(error).
// Does not null-terminate output.
PUBLIC_FN ssize_t hex_decode(char *in, ssize_t in_len, char *out) {
  char *out_ = out;
  if (in_len == -1) in_len = strlen(in);
  assert(in_len >= 0);
  if (in_len&1) return -1; /* n%2 must be 0 */
  char *in_end = in + in_len;
  while (in < in_end) {
    unsigned char hi = unhexchar(*(in++));
    unsigned char lo = unhexchar(*(in++));
    if (hi == 255 || lo == 255) return -1;
    *(out++) = (hi<<4) | lo;
  }
  return out_ - out;
}
